import asyncio
import re
from datetime import datetime, timedelta
from enum import Enum
from typing import List

import typer

from .get import get_reports
from .sanitize import sanitize_metar, sanitize_taf

app = typer.Typer()


def remove_white_spaces(reports: List[str]):
    sanitized_reports: List[str] = []
    for report in reports:
        report = re.sub(r"\s{2,}|\n+|\t+", " ", report)
        report = report.strip()
        sanitized_reports.append(report)
    return sanitized_reports


class ReportType(str, Enum):
    SA = "SA"
    SP = "SP"
    FT = "FT"
    FC = "FC"
    ALL = "ALL"


@app.command()
def main(
    icao: str = typer.Argument(
        default="MROC",
        help="The ICAO code of the station to request, e.g. MROC for Int. Airp. Juan Santamaría",
    ),
    report_type: ReportType = typer.Option(
        ReportType.SA,
        "--type",
        "-t",
        help="""Type of report to request.
        SA -> METAR,
        SP -> SPECI,
        FT -> TAF (long),
        FC -> TAF (short),
        ALL -> All types""",
    ),
    init_date: datetime = typer.Option(
        "2006-01-01T00:00:00",
        "--init",
        "-i",
        help="The initial UTC date and time to request the reports.",
    ),
    final_date: datetime = typer.Option(
        None,
        "--final",
        "-f",
        help=(
            "The final UTC date and time to request the reports. "
            "Defaults to `init` + 30 days, 23 hours and 59 minutes."
        ),
    ),
    filename: str = typer.Option(
        "metar.txt",
        "--file",
        "-F",
        help="The filename to write the reports on disk. Default will be changed",
    ),
    one_line: bool = typer.Option(
        False,
        "--one-line",
        "-o",
        is_flag=True,
        help=(
            "Remove white spaces in the reports. "
            "If True reports will be written in one line."
        ),
    ),
    sanitize: bool = typer.Option(
        False,
        "--sanitize",
        "-s",
        help="Sanitize the report to use in TAF verification program.",
    ),
) -> None:
    if init_date > datetime.today():
        typer.echo(f"Initial date and time must be older than current date and time.")
        return

    if report_type == "FT" or report_type == "FC":
        filename = "taf.txt"
    elif report_type == "ALL":
        filename = "report.txt"
    elif report_type == "SP":
        filename = "speci.txt"
    else:
        pass

    if final_date is None:
        final_date = init_date + timedelta(days=30, hours=23, minutes=59)
    typer.echo(f"Request from {init_date} to {final_date}.")

    reports: List[str] = []
    try:
        reports = asyncio.run(
            get_reports(icao.upper(), report_type, str(init_date), str(final_date))
        )
    except Exception as e:
        typer.echo(f"{e}.".capitalize())
    else:
        if one_line:
            reports = remove_white_spaces(reports)

    with open(f"./{filename}", "w") as f:
        for report in reports:
            if sanitize:
                if report_type in ["SA", "SP"]:
                    report = sanitize_metar(report, icao)
                elif report_type in ["FC", "FT"]:
                    report = sanitize_taf(report, icao)
            f.write(report + "\n")

    report_filename = filename.replace(".txt", "")
    if len(reports) > 0:
        typer.echo(f"{len(reports)} {report_filename.upper()} requested succesfully.")
    if len(reports) == 0:
        typer.echo(f"No {report_filename.upper()} requested.")


if __name__ == "__main__":
    app()



import asyncio
from datetime import datetime
from typing import Dict, List, Optional

import aiohttp
from bs4 import BeautifulSoup
from pydantic import validator
from pydantic.dataclasses import dataclass
from typing_extensions import Literal

today = datetime.today()


@dataclass
class DateInitPayload:
    date: Optional[datetime]

    @validator("date")
    def parse_date(cls, v: str):
        if isinstance(v, str):
            try:
                date = datetime.strptime(v, "%Y-%m-%dT%H:%M")
            except ValueError:
                return datetime.today()
            else:
                return date

        return v

    @property
    def to_dict(self) -> Dict[str, str]:
        return {
            "ano": f"{self.date.year}",
            "mes": f"{self.date.month:02d}",
            "day": f"{self.date.day:02d}",
            "hora": f"{self.date.hour:02d}",
        }


class DateFinalPayload(DateInitPayload):
    @property
    def to_dict(self) -> Dict[str, str]:
        return {
            "anof": f"{self.date.year}",
            "mesf": f"{self.date.month:02d}",
            "dayf": f"{self.date.day:02d}",
            "horaf": f"{self.date.hour:02d}",
            "minf": f"{self.date.minute:02d}",
        }


@dataclass
class Payload:
    icao: str
    type_: Literal["ALL", "SA", "SP", "FT", "FC"]
    ord_: Literal["REV", "DIR"]
    nil: Literal["SI", "NO"]
    fmt: Literal["html", "txt"]

    @validator("icao")
    def must_be_a_valid_icao_code(cls, v: str) -> str:
        assert len(v) == 4, "ICAO code must be 4 characters lenght"
        assert v.isalnum(), "invalid characters in ICAO code"
        return v

    @property
    def to_dict(self) -> Dict[str, str]:
        return {
            "lugar": self.icao,
            "tipo": self.type_,
            "ord": self.ord_,
            "nil": self.nil,
            "fmt": self.fmt,
            "enviar": "Ver",
        }


def validate_dates(init: datetime, final: datetime):
    delta = final - init
    assert final > init, "final date must be most recent than init date"
    assert delta.days <= 31, "only requests of 31 days are valid"


OGIMET_METAR_URL = "http://ogimet.com/display_metars2.php"


async def get_data(payload: Dict[str, str]) -> List[str]:
    async with aiohttp.request("GET", OGIMET_METAR_URL, params=payload) as resp:
        data = await resp.text()
        soup = BeautifulSoup(data, "html5lib")
        pre_tags = soup.findAll("pre")

        return [tag.string for tag in pre_tags]


async def get_reports(icao: str, type_: str, init: str, final: str) -> None:
    # Init dates
    init_date = DateInitPayload(date=init)
    final_date = DateFinalPayload(date=final)
    # Validate dates
    validate_dates(init_date.date, final_date.date)
    # Init payload
    payload_instance = Payload(icao=icao, type_=type_, ord_="DIR", nil="SI", fmt="html")
    payload = payload_instance.to_dict
    # Update payload with valid dates
    payload.update(init_date.to_dict)
    payload.update(final_date.to_dict)
    # Getting the data
    reports = await get_data(payload)
    # Return the reports as a List[str]
    return reports


if __name__ == "__main__":
    reports = asyncio.run(get_reports("FT", "2015-01-01T00:00", "2015-01-31T23:00"))
    for report in reports:
        print(report)



import re
from datetime import datetime, timedelta

# from metar import Metar
import typer


def _sanitize_type_metar(metar: str, icao_code: str):
    metar = re.sub(r"(METAR\s+)?SPECI(\s+METAR)?", "METAR", metar)
    metar = metar.replace("COR", "")
    metar = metar.replace("ZZ", "Z")

    if metar.count("METAR") == 0:
        icao = icao_code.upper()
        metar = metar.replace(icao, f"METAR {icao}")

    return metar


def _sanitize_wind(report: str):
    report = report.replace("KTKT", "KT")

    format = r"\s(?P<dir>\d{3})(?P<speed>\d{2})(?P<g>G?)(?P<gust>\d{2}?)(?P<units>(KT|MPS|KTS)?)\s"
    pattern = re.compile(format)

    matches = re.search(pattern, report)

    if matches:
        if matches.group("g") is None and matches.group("gust") is not None:
            correct_format = f'{matches.group("dir")}{matches.group("speed")}G{matches.group("gust")}{matches.group("units")}'
            report = re.sub(format, correct_format, report)

    report = re.sub(r"\d{3}V\d{3}", "", report)

    return report


def _sanitize_weather(report: str):
    report = re.sub(r"DZ|SHRA", "RA", report)
    report = re.sub(r"BR|HZ|[BP][CR]FG", "FG", report)
    report = re.sub(r"(FG\s){2,}", "FG ", report)
    report = re.sub(r"VC[A-Z][A-Z]", "", report)
    report = re.sub(r"\+|-", "", report)

    return report


def _sanitize_visibility(report: str):
    report = report.replace("CAVOK", "9999")
    report = re.sub(r"9999\s{2,}|99999", "9999 ", report)
    report = re.sub(r"\s\d{3,4}[NS]([ENSW])?([EW])?", "", report)
    report = re.sub(
        r"R\d{2}([A-Z])?/(P|M)?\d{4}(V(P|M)?\d{4})?(FT)?([NDU])?", "", report
    )

    return report


def _sanitize_cloud(report: str):
    report = re.sub(r"VV///|NSC", "", report)
    report = re.sub(r"\s(GEW|FWE|HEW|FE[A-Z]W|F[A-Z]EW)", " FEW", report)
    report = re.sub(r"\s(ACT|STC|DCT|SC[A-Z]T|S[A-Z]CT)", " SCT", report)
    report = re.sub(r"\s(VKN|BNK|NNK|BK[A-Z]N|B[A-Z]KN)", " BKN", report)
    report = re.sub(r"\s(IVC|PVC|OV[A-Z]C|O[A-Z]VC)", " OVC", report)
    report = re.sub(r"TCU|TUC|CB", "", report)

    return report


def _remove_suplementary_info(metar: str):
    metar = re.sub(
        r"\sWS\s?(R?(/|\s+)?\d{2,}|(ALL)?\s?R[WNU]+Y\s?\d{2,}).*", "=", metar
    )
    metar = re.sub(r"\sRE[A-Z]{2,}.*", "=", metar)

    return metar


def _remove_rmk_and_trend_indicator(metar: str):
    metar = re.sub(r"\s?RMK.+", "=", metar)
    metar = re.sub(
        r"\s?(BECMG|BECMG(0|O|\??\s?\?+)|VECMG|BCMG|BECMG/|BECM/|BCMG/|BECOMG|BECGM|BECM\sG)(.+)?",
        "=",
        metar,
    )
    metar = re.sub(
        r"\s?(NOSIG|NOPSIG|NSIG|NPSIG|NOZIG|NOPZIG|NOSOG|MOSIG|NISIG|NOSGI|NOSI\sG|NOSIG0|NSOIG|NOSIIG)(.+)?",
        "=",
        metar,
    )
    metar = re.sub(r"\s?(TEMPO|TMPO|TEMPOO|TMEPO|TEMOP|TEMO|TEMP\sO)(.+)?", "=", metar)

    return metar


# TODO: Enable parsing of reports when possible
# def parse_metar(metar: str):
#     code = metar.replace("=", "")
#     code = re.sub(r"^\d{12}\s", "", code)
#     code = code.strip()

#     date = datetime.strptime(metar[:12], "%Y%m%d%H%M")

#     try:
#         if not "NIL" in metar:
#             metar_obj = Metar.Metar(code, month=date.month, year=date.year)
#     except Metar.ParserError as e:
#         typer.echo(f"{e}")


def sanitize_metar(metar: str, icao_code: str):
    metar = _remove_rmk_and_trend_indicator(metar)
    metar = _remove_suplementary_info(metar)
    metar = _sanitize_type_metar(metar, icao_code)
    metar = _sanitize_wind(metar)
    metar = _sanitize_visibility(metar)
    metar = _sanitize_weather(metar)
    metar = _sanitize_cloud(metar)

    metar = re.sub(r"={2,}", "=", metar)
    metar = re.sub(r"\s{2,}", " ", metar)

    #    parse_metar(metar)

    return metar


def _sanitize_type_taf(taf: str, icao_code: str):
    taf = re.sub(r"COR|AMD", "", taf)

    if taf.count("TAF") == 0:
        icao = icao_code.upper()
        taf = taf.replace(icao, f"TAF {icao}")

    return taf


def _sanitize_validity_period(taf: str):
    format = r"^\d{12}\s"
    pattern = re.compile(format)
    match = re.match(pattern, taf)

    if match:
        date = datetime.strptime(taf[:12], "%Y%m%d%H%M")
        format = r"\s\d{2}00/\d{2}00\s"
        pattern = re.compile(format)
        period_match = re.search(pattern, taf)

        if period_match:
            period_date = date + timedelta(days=1)
            period_day = (
                f"{period_date.day}" if period_date.day >= 10 else f"0{period_date.day}"
            )
            taf = re.sub(format, f" {period_day}00/{period_day}24 ", taf)

    return taf


def _sanitize_change_group_code(taf: str):
    taf = re.sub(
        r"BECMG0|VECMG|BCMG|BECMG/|BECM/|BCMG/|BECOMG|BECGM|BECM\sG", "BECMG", taf
    )
    taf = re.sub(
        r"NOPSIG|NSIG|NPSIG|NOZIG|NOPZIG|NOSOG|MOSIG|NISIG|NOSGI|NOSI\sG|NOSIG0|NSOIG|NOSIIG",
        "NOSIG",
        taf,
    )
    taf = re.sub(r"TMPO|TEMPOO|TMEPO|TEMOP|TEMO|TEMP\sO", "TEMPO", taf)

    return taf


def sanitize_taf(taf: str, icao_code: str):
    taf = _sanitize_type_taf(taf, icao_code)
    taf = _sanitize_validity_period(taf)
    taf = _sanitize_wind(taf)
    taf = _sanitize_visibility(taf)
    taf = _sanitize_weather(taf)
    taf = _sanitize_cloud(taf)
    taf = _sanitize_change_group_code(taf)

    return taf



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['get_metars']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.8.1,<4.0.0',
 'beautifulsoup4>=4.11.1,<5.0.0',
 'html5lib>=1.1,<2.0',
 'pydantic>=1.9.1,<2.0.0',
 'requests>=2.27.1,<3.0.0',
 'typer>=0.4.1,<0.5.0']

entry_points = \
{'console_scripts': ['tafver = get_metars.__main__:get_metars']}

setup_kwargs = {
    'name': 'get-metars',
    'version': '0.1.0',
    'description': "Simple command line tool to download METAR's and TAF's for a given station and store them in text files.",
    'long_description': None,
    'author': 'diego-garro',
    'author_email': 'diego.garromolina@yahoo.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/diego-garro/tafver-metars',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
